#!/usr/bin/python
# -*- coding: UTF-8 -*-

import binascii
import hashlib
import hmac
import json
from datetime import datetime

# python 2 and python 3 compatibility library
from six.moves.urllib.parse import quote
from six.moves.urllib.parse import urlparse

from dli.rest import ApiClient


class AkskApiClient(ApiClient):
    def __init__(self, ak, sk, region, project_id, service='dli', **kwargs):
        self._ak = ak
        self._sk = sk
        self._region = region
        self._project_id = project_id
        self._service = service
        ApiClient.__init__(self, **kwargs)

    def request(self, method, url, query_params=None, headers=None,
                post_params=None, body=None, _preload_content=True, _request_timeout=None):
        req_headers = headers

        if 'X-Auth-Token' in req_headers:
            del req_headers['X-Auth-Token']

        req_headers['X-Project-Id'] = self._project_id

        singer = Signer(self._ak, self._sk, self._region, self._service)
        singer.sign(method, url, query_params, req_headers, body)

        return ApiClient.request(self, method, url,
                                 query_params=query_params,
                                 headers=req_headers,
                                 post_params=post_params, body=body,
                                 _preload_content=_preload_content,
                                 _request_timeout=_request_timeout)


BasicDateFormat = "%Y%m%dT%H%M%SZ"
BasicDateFormatShort = "%Y%m%d"


def urlencode(s):
    return quote(s, safe='~')


# Build a CanonicalRequest from a regular request string
# See http://docs.aws.amazon.com/general/latest/gr/sigv4-create-canonical-request.html
def build_canonical_request(method, path, query, headers, body):
    if 'x-sdk-content-sha256' in headers.keys():
        hexencode = headers['x-sdk-content-sha256']
    else:
        hexencode = encode_hash_body(body.encode('utf-8'))
    # print ("body hex=%s" %(hexencode))
    return "%s\n%s\n%s\n%s\n%s\n%s" % (
        method, canonical_uri(path), canonical_querystring(query),
        canonical_headers(headers), signed_headers(headers), hexencode)


def signed_headers(headers):
    a = []
    for key in headers:
        a.append(key.lower())
    a.sort()
    return ";".join(a)


def canonical_headers(headers):
    a = []
    for key in headers:
        value = headers[key]
        keyEncoded = key.lower()
        a.append(keyEncoded + ":" + value.strip())
    a.sort()
    return '\n'.join(a) + "\n"


# encode_hash_body returns hexcode of sha256
def encode_hash_body(body):
    sha256 = hashlib.sha256()
    sha256.update(body)
    return sha256.hexdigest()


def canonical_uri(uri):
    pattens = uri.split('/')
    uri = []
    for v in pattens:
        if v == "" or v == ".":
            continue
        elif v == '..':
            if len(uri) > 0:
                uri.pop()
        else:
            uri.append(urlencode(v))
    urlpath = "/"
    if len(uri) > 0:
        urlpath = urlpath + "/".join(uri) + "/"  # always end with /
    # r.uri = urlpath
    return urlpath


def canonical_querystring(query):
    a = []
    for key in query:
        value = query[key]
        if value == "":
            kv = urlencode(key)
        else:
            kv = urlencode(key) + "=" + urlencode(value)
        a.append(kv)
    a.sort()
    return '&'.join(a)


# Return the Credential Scope. See http://docs.aws.amazon.com/general/latest/gr/sigv4-create-string-to-sign.html
def build_credential_scope(time, region, service):
    return "%s/%s/%s/sdk_request" % (datetime.strftime(time, BasicDateFormatShort), region, service)


# Create a "String to Sign". See http://docs.aws.amazon.com/general/latest/gr/sigv4-create-string-to-sign.html
def build_string_to_sign(canonical_request, credential_scope, t):
    sha256 = hashlib.sha256()
    sha256.update(canonical_request.encode('utf-8'))
    bytes = sha256.hexdigest()
    return "%s\n%s\n%s\n%s" % ('SDK-HMAC-SHA256', datetime.strftime(t, BasicDateFormat), credential_scope, bytes)


def generate_signing_key(app_secret, region, service, t):
    key = ('SDK' + app_secret).encode('utf-8')
    dateStamp = datetime.strftime(t, BasicDateFormatShort)
    for d in [dateStamp, region, service, "sdk_request"]:
        key = hmacsha256(key, d)
        # print ("%s:%s" %(d, key.hex()))
    return key


def hmacsha256(key_byte, message):
    return hmac.new(key_byte, message.encode('utf-8'), digestmod=hashlib.sha256).digest()


# Create the HWS Signature. See http://docs.aws.amazon.com/general/latest/gr/sigv4-calculate-signature.html
def sign_string(string_to_sign, signing_key):
    hm = hmacsha256(signing_key, string_to_sign)
    return binascii.hexlify(hm).decode()


def build_signed_headers(headers):
    a = []
    for key in headers:
        a.append(key.lower())
    a.sort()
    return ";".join(a)


# Get the finalized value for the "Authorization" header.  The signature
# parameter is the output from SignStringToSign
def build_auth_header_value(signature, app_key, credential_scope, signed_headers):
    return "%s Credential=%s/%s, SignedHeaders=%s, Signature=%s" % (
        'SDK-HMAC-SHA256', app_key, credential_scope, signed_headers, signature)


class Signer(object):
    def __init__(self, ak, sk, region, service='dli'):
        self._ak = ak
        self._sk = sk
        self._region = region
        self._service = service

    def sign(self, method, path, query, headers, body):
        header_time = headers.get('X-Sdk-Date')
        if header_time is None:
            time = datetime.utcnow()
            headers['X-Sdk-Date'] = datetime.strftime(time, BasicDateFormat)
        else:
            time = datetime.strptime(header_time, BasicDateFormat)

        content = ''
        if body:
            content = json.dumps(body)

        url = urlparse(path)
        uri = url.path

        req_query = {}
        if query:
            for k, v in query:
                req_query[k] = str(v)

        # ************* TASK 1: CREATE A CANONICAL REQUEST *************
        # http://docs.aws.amazon.com/general/latest/gr/sigv4-create-canonical-request.html
        canonical_request = build_canonical_request(method, uri, req_query, headers, content)

        # ************* TASK 2: CREATE THE STRING TO SIGN*************
        # Match the algorithm to the hashing algorithm you use, either SHA-1 or
        credential_scope = build_credential_scope(time, self._region, self._service)
        string_to_sign = build_string_to_sign(canonical_request, credential_scope, time)

        # ************* TASK 3: CALCULATE THE SIGNATURE *************
        # Create the signing key using the function defined above.
        signing_key = generate_signing_key(self._sk, self._region, self._service, time)
        signature = sign_string(string_to_sign, signing_key)

        # ************* TASK 4: ADD SIGNING INFORMATION TO THE REQUEST *************
        # Put the signature information in a header named Authorization.
        signed_headers = build_signed_headers(headers)
        auth_header = build_auth_header_value(signature, self._ak, credential_scope, signed_headers)

        headers['Authorization'] = auth_header
