#!/usr/bin/python
# -*- coding: UTF-8 -*-

"""
    Huawei DLI Python SDK
    The DliClient class of DLI Python SDK
"""

import time

from dli.auth import AkskApiClient
from dli.column import Column
from dli.database import Database
from dli.dli_info import DliInfo
from dli.exception import DliException
from dli.job import SqlJob
from dli.job import SparkBatchJob
from dli.queue import Queue
from dli.resource import Resources
from dli.resource import PackageResource
from dli.resource import GroupResource
from dli.rest.apis.rest_api import RestApi
from dli.rest.apis.token_auth_api import TokenAuthApi
from dli.rest.api_client import ApiClient
from dli.rest.configuration import Configuration
from dli.rest.models import *
from dli.rest.rest import ApiException
from dli.table import Table, TableSchema


class DliClient(object):
    MAX_PAGE_SIZE = 100

    def __init__(self, **kwargs):
        """
        Init a DLI client

        :param str auth_mode: 'token' or 'aksk' mode.
                            use 'token' mode should input account, user and password
                            use 'aksk' mode should input ak, sk
        :param str region: The region, for example, cn-north-1
        :param str project_id: On the management console, displayed My Credential page,
                                click Project List to query the project ID,
                                for example, 5a3314075bfa49b9ae360f4ecd333695.
        :param str token: if not null, will not need set auth_mode
        :param str domain: domain, default: myhuaweicloud.com
        :param str account: Account name
        :param str user: IAM user name
        :param str password: Password
        :param str ak: Access Key
        :param str sk: Secret Key
        :param str proxy: proxy address, for example: https://192.168.64.187:3128
        """

        self._dli_info = DliInfo(**kwargs)
        self._conf = Configuration()
        self._conf.host = 'https://' + self.dli_info.dli
        self._conf.verify_ssl = True
        self._conf.proxy = kwargs.get('proxy')
        self._auth_mode = kwargs.get('auth_mode')

        self._needGetToken = True

        if self.dli_info.token is not None:
            self._needGetToken = False
            self._token = self.dli_info.token
            self._rest_api = RestApi()

        if self._needGetToken and self._auth_mode == 'token':
            self._rest_api = RestApi()

            get_token_api = ApiClient(host='https://' + self.dli_info.iam)
            self._token_api = TokenAuthApi(api_client=get_token_api)
            (self._token_expires_time, self._token) = self.get_token()
            self._token_expires_tm = \
                time.mktime(time.strptime(self._token_expires_time,
                                          "%Y-%m-%dT%H:%M:%S.%fZ"))
        if self._auth_mode == 'aksk':
            aksk_api_client = AkskApiClient(self.dli_info.ak,
                                            self.dli_info.sk,
                                            self.dli_info.region,
                                            self.dli_info.project_id,
                                            host=self._conf.host)
            self._rest_api = RestApi(api_client=aksk_api_client)

    @property
    def dli_info(self):
        return self._dli_info

    @property
    def rest_api(self):
        return self._rest_api

    @property
    def token_expires_time(self):
        return self._token_expires_time

    @property
    def token(self):
        if not self._needGetToken:
            return self._token

        if self._auth_mode != 'token':
            return None

        if self.is_token_expired():
            (self._token_expires_time, self._token) = self.get_token()
            self._token_expires_tm = \
                time.mktime(time.strptime(self._token_expires_time,
                                          "%Y-%m-%dT%H:%M:%S.%fZ"))

        return self._token

    def get_token(self):
        try:
            (resp_body, http_code, resp_headers) \
                = self._token_api.get_token_with_http_info(self._build_token_info())
        except ApiException as e:
            raise DliException(api_exception=e)

        return resp_body.token.expires_at, resp_headers['X-Subject-Token']

    def is_token_expired(self):
        now = time.time()
        return now >= self._token_expires_tm

    def _build_token_info(self):
        domain = Domain()
        domain.name = self.dli_info.account

        user = User()
        user.domain = domain
        user.name = self.dli_info.user
        user.password = self.dli_info.pwd

        pwd = Password()
        pwd.user = user

        identity = Identity()
        identity.password = pwd
        identity.methods = ['password']

        project = Project()
        project.id = self.dli_info.project_id

        scope = Scope()
        scope.project = project

        auth = Auth()
        auth.identity = identity
        auth.scope = scope

        token_info = TokenAuthInfo()
        token_info.auth = auth

        return token_info

    def list_queues(self):
        """
        This method is used to list all queues under the project.

        :return: list
                The list of all queues.
        :raises: DliException
        """
        try:
            resp = self._rest_api.list_all_queues(self.dli_info.project_id,
                                                  x_auth_token=self.token,
                                                  queue_type="all")
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        queues = []
        for queue_info in resp.queues:
            queues.append(Queue(queue_info))

        return queues

    def get_queue(self, queue_name):
        """
        This method is used to get a queue by queue name.

        :param str queue_name: Queue name
        :return: Queue
                If the Queue do not exist, will return None
        :raises: DliException
        """

        queues = self.list_queues()
        for queue in queues:
            if queue.name == queue_name:
                return queue

    def list_databases(self):
        """
        This method is used to query the information about all the databases.

        :return: list
                The list of all databases.
        :raises: DliException
        """

        try:
            resp = self._rest_api.list_all_databases(self.dli_info.project_id,
                                                     x_auth_token=self.token)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        dbs = []
        for db_info in resp.databases:
            dbs.append(Database(db_info, self))

        return dbs

    def get_database(self, db_name):
        """
        This method is used to get a database by database name.

        :param str db_name: database name
        :return: Database
                If the Database do not exist, will return None
        :raises: DliException
        """

        dbs = self.list_databases()
        for db in dbs:
            if db.name == db_name:
                return db

    def create_database(self, db_name, description=None):
        """
        This method is used to create a database, make sure database name is not exist.

        :param str db_name: Database name
        :param str description: The description of the database
        :return: Database
        """

        db_info = DatabaseInfo()
        db_info.database_name = db_name
        db_info.description = description
        try:
            resp = self._rest_api.create_database(self.dli_info.project_id,
                                                  db_info,
                                                  x_auth_token=self.token)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return self.get_database(db_name)

    def create_database_with_eps(self, db_name, enterprise_project_id, description=None):
        """
        This method is used to create a database, make sure database name is not exist, enterprise_project_id is exist.

        :param str db_name: Database name
        :param str enterprise_project_id: EnterpriseProjectId
        :param str description: The description of the database
        :return: Database
        """

        db_info = DatabaseInfo()
        db_info.database_name = db_name
        db_info.description = description
        db_info.enterprise_project_id = enterprise_project_id

        try:
            resp = self._rest_api.create_database(self.dli_info.project_id,
                                                  db_info,
                                                  x_auth_token=self.token)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return self.get_database(db_name)

    def delete_database(self, db_name):
        """
        This method is used to delete an empty database. If there are tables
        in the database to be deleted, delete all tables first.

        :param str db_name: Database name
        :return: None
        """

        kwargs = {
            'project_id': self.dli_info.project_id,
            'x_auth_token': self.token,
            'database_name': db_name
        }

        try:
            resp = self.rest_api.delete_database(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

    def list_tables(self, db_name, with_detail=False, keyword=None):
        """
        This method is used to query information about tables that meet the filtering
        criteria or all the tables in the specified database.

        :param str db_name: database name
        :param bool with_detail: Whether to get the details information
                of the tables (owner, size, etc.).
        :param str keyword: The keyword of the filter table name.
        :return: list
                The list of all tables in the specified database.
        """

        kwargs = {
            'x_auth_token': self.token,
            'page_size': self.MAX_PAGE_SIZE,
            'with_detail': with_detail
        }

        if keyword:
            kwargs['keyword'] = keyword

        tables = []
        for i in range(10):
            try:
                resp = self._rest_api.list_all_tables(self.dli_info.project_id,
                                                      db_name, current_page=i + 1,
                                                      **kwargs)
            except ApiException as e:
                raise DliException(api_exception=e)
            if not resp.is_success:
                raise DliException(msg=resp.message)

            for table_info in resp.tables:
                tables.append(Table(table_info.table_name, db_name, table_info))

            if resp.table_count < self.MAX_PAGE_SIZE:
                break

        return tables

    def list_tables_page(self, db_name, page_size, current_page, with_detail=False, keyword=None, table_type=None):
        """
        This method is used to query information about tables that meet the filtering
        criteria or all the tables by page in the specified database.

        :param str db_name: database name
        :param int page_size: the page size to get tables, should be in range 1 to 100
        :param int current_page: current page to get tables
        :param bool with_detail: Whether to get the details information
                 of the tables (owner, size, etc.).
        :param str keyword: The keyword of the filter table name.
        :param str table_type: Filter by table type.(EXTERNAL,MANAGED,VIEW)
        :return: list
                 The list of all tables in the specified database.
        """

        kwargs = {
            'x_auth_token': self.token,
            'page_size': page_size,
            'current_page': current_page,
            'with_detail': with_detail
        }

        if keyword:
            kwargs['keyword'] = keyword
        # SDK tableType is difference from rest api,this should convert to the real type in metadata.
        # EXTERNAL -> MANAGED_TABLE, MANAGED -> EXTERNAL_TABLE, VIEW -> VIRTUAL_VIEW
        if table_type:
            if table_type == "EXTERNAL":
                kwargs['table_type'] = "EXTERNAL_TABLE"
            elif table_type == "MANAGED":
                kwargs['table_type'] = "MANAGED_TABLE"
            elif table_type == "VIEW":
                kwargs['table_type'] = "VIRTUAL_VIEW"
            else:
                raise DliException(msg="Invalid table type for list tables: %s" % table_type)
        tables = []
        try:
            resp = self._rest_api.list_all_tables(self.dli_info.project_id,
                                                  db_name,
                                                  **kwargs)

        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        for table_info in resp.tables:
            tables.append(Table(table_info.table_name, db_name, table_info))
        return resp.table_count, tables

    def get_table(self, db_name, tbl_name):
        """
        This method is used to get the table in the specified database by table name.

        :param str db_name: database name
        :param str tbl_name: table name
        :return: Table
                 If the table do not exist, will return None
        """

        tables = self.list_tables(db_name, with_detail=True, keyword=tbl_name)
        for table in tables:
            if tbl_name == table.name:
                return table

    def get_table_schema(self, db_name, tbl_name):
        """
        This method is used to describe schema information in the specified table.

        :param str db_name: database name
        :param str tbl_name: table name
        :return: TableSchema
        """

        kwargs = {
            'project_id': self.dli_info.project_id,
            'database_name': db_name,
            'table_name': tbl_name,
            'x_auth_token': self.token
        }

        try:
            resp = self.rest_api.describe_table(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        sort_columns = resp.sort_columns
        columns = []
        for col_info in resp.columns:
            columns.append(Column(col_info.column_name,
                                  col_info.type,
                                  col_info.description,
                                  col_info.is_partition_column))

        return TableSchema(tbl_name, columns, sort_columns)

    def create_dli_table(self, db_name, schema, description=None):
        """
        This method is used to create DLI table.

        :param str db_name: database name
        :param TableSchema schema: the scheme for the table
        :param str description: table description
        :return: Table
        """

        cols = []
        for col in schema.columns:
            cols.append(col.column_info)

        sort_cols = None
        desc = None
        if schema.sort_columns is not None:
            sort_cols = schema.sort_columns
        if description is not None:
            desc = description

        tbl_info = TableInfo(table_name=schema.table_name,
                             data_location='DLI',
                             columns=cols,
                             sort_columns=sort_cols,
                             description=desc)
        kwargs = {
            'project_id': self.dli_info.project_id,
            'database_name': db_name,
            'table_info': tbl_info,
            'x_auth_token': self.token
        }

        try:
            resp = self.rest_api.create_table(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return self.get_table(db_name, schema.table_name)

    def create_obs_table(self, db_name, schema, data_path,
                         data_type, description=None, options=None):
        """
        This method is used to create OBS table..

        :param str data_type: Type of data in the new OBS table. Currently, the parameter
                                value can be parquet, ORC, CSV, or JSON.
        :param str data_path: Storage path of data in the new OBS table, which
                                must be a path on OBS and must begin with s3a.
                                For examples: s3a://data/data
        :param str db_name: database name
        :param TableSchema schema: the scheme for the table
        :param str description: table description
        :param dict options: The dict options of the data type, for the detailed information refer to
                            https://support.huaweicloud.com/api-uquery/uquery_02_0100.html
                            For examples:
                            {
                                "with_column_header": true,
                                "delimiter": ",",
                                "quote_char": "\"",
                                "escape_char": "\\",
                                "date_format": "yyyy/MM/dd",
                                "timestamp_format": "yyyy/MM/dd hh:mm:ss"
                            }
        :return: Table
        """

        cols = []
        for col in schema.columns:
            cols.append(col.column_info)

        desc = None
        if description:
            desc = description

        with_column_header = None
        delimiter = None
        quote_char = None
        escape_char = None
        date_format = None
        timestamp_format = None
        if options:
            with_column_header = options.get('with_column_header')
            delimiter = options.get('delimiter')
            quote_char = options.get('quote_char')
            date_format = options.get('date_format')
            timestamp_format = options.get('timestamp_format')
            escape_char = options.get('escape_char')

        tbl_info = TableInfo(table_name=schema.table_name,
                             data_location='OBS',
                             columns=cols,
                             description=desc,
                             data_path=data_path,
                             data_type=data_type,
                             with_column_header=with_column_header,
                             delimiter=delimiter,
                             quote_char=quote_char,
                             escape_char=escape_char,
                             date_format=date_format,
                             timestamp_format=timestamp_format)
        kwargs = {
            'project_id': self.dli_info.project_id,
            'database_name': db_name,
            'table_info': tbl_info,
            'x_auth_token': self.token
        }
        try:
            resp = self.rest_api.create_table(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return self.get_table(db_name, schema.table_name)

    def delete_table(self, db_name, table_name):
        """
        This method is used to delete a specified table.

        :param str db_name: Name of the database where the table to be deleted resides.
        :param str table_name: Name of the table to be deleted.
        :return: None
        """

        kwargs = {
            'project_id': self.dli_info.project_id,
            'database_name': db_name,
            'table_name': table_name,
            'x_auth_token': self.token
        }

        try:
            resp = self.rest_api.delete_table(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return

    def _submit_sql_job(self, sql, db_name=None, queue_name='default', options=None):
        conf_list = None
        if options:
            conf_list = []
            for key, value in options.items():
                conf_list.append(key + '=' + value)

        sql_req = SQLJobRequest(currentdb=db_name, sql=sql,
                                conf=conf_list, queue_name=queue_name)

        kwargs = {
            'project_id': self.dli_info.project_id,
            'x_auth_token': self.token,
            'sql_job_info': sql_req
        }

        try:
            resp = self.rest_api.execute(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return resp

    def async_execute_sql(self, sql, db_name=None, queue_name='default', options=None):
        """
        Execute SQL job in asynchronous mode.
        This method is used to submit SQL statements to the server cluster.
        The job contains the following types: DDL, DCL, IMPORT, EXPORT, QUERY and INSERT.

        :param str sql: SQL statement
        :param str queue_name: Queue name for the SQL execute, use 'default' queue for default
        :param str db_name: Database name of the table belongs to, if the SQL statement have
                             database name, it can set to None.
        :param dict options: User-defined parameter that applies to the job
                             examples:
                             {'dli.sql.cbo.enabled': True,
                              'dli.sql.syncMode.enabled': True}
        :return: SqlJob
        """

        resp = self._submit_sql_job(sql, db_name, queue_name, options)

        return SqlJob(resp.job_id, resp.job_type, self)

    def execute_sql(self, sql, db_name=None, queue_name='default', options=None, retry_times=1000):
        """
        Execute SQL job
        This method is used to submit SQL statements to the server cluster.
        The job contains the following types: DDL, DCL, IMPORT, EXPORT, QUERY and INSERT.
        This method is a synchronous mode, blocking until SQL execution is completed.

        :param str sql: SQL statement
        :param str queue_name: Queue name for the SQL execute, use 'default' queue for default
        :param str db_name: Database name of the table belongs to, if the SQL statement have
                             database name, it can set to None.
        :param dict options: User-defined parameter that applies to the job
                             examples:
                             {'dli.sql.cbo.enabled': True,
                              'dli.sql.syncMode.enabled': True}
        :return: SqlJob
        """

        resp = self._submit_sql_job(sql, db_name, queue_name, options)

        if resp.job_type == 'QUERY':
            return self._proc_query_sql(resp)
        elif resp.job_type == 'DDL' or resp.job_type == 'DCL':
            # DDL and DCL is running is sync mode
            if resp.job_mode == 'async' or resp.job_mode == 'ASYNC':
                job_status = self.wait_for_finish(resp.job_id, retry_times)
                return SqlJob(resp.job_id, resp.job_type, self, status=job_status)
            else:
                return self._proc_ddl_dcl_sql(resp)
        else:
            # IMPORT, EXPORT and INSERT statement
            job_status = self.wait_for_finish(resp.job_id, retry_times)
            return SqlJob(resp.job_id, resp.job_type, self, status=job_status)

    def cancel_sql(self, job_id):
        """

        :param str job_id: ID of a job returned after a job is submitted.
        :return:
        """

        try:
            if not job_id:
                raise DliException(msg="No job in progress, cancel job failed")
            resp = self._rest_api.cancel_job(self.dli_info.project_id,
                                             job_id, x_auth_token=self.token)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

    def list_sql_jobs(self):
        """
        This method is used to list all sql jobs under the project.

        :return: list
                The list of all jobs.
        :raises: DliException
        """
        sql_jobs = []
        try:
            job_info_size = 101
            i = 1
            while job_info_size >= 100:
                job_infos = self.list_sql_jobs_page(100, i)
                job_info_size = len(job_infos)
                i += 1
                sql_jobs = sql_jobs + job_infos
        except ApiException as e:
            raise DliException(api_exception=e)

        return sql_jobs

    def list_sql_jobs_page(self, page_size, current_page):
        """
        This method is used to list sql jobs by page under the project.

        :return: list
                The list of all jobs.
        :raises: DliException
        """
        try:
            resp = self._rest_api.list_sql_jobs(self.dli_info.project_id,
                                                x_auth_token=self.token,
                                                page_size=page_size,
                                                current_page=current_page)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        sql_jobs = []
        for sql_job_info in resp.jobs:
            sql_jobs.append(sql_job_info)
        return sql_jobs

    def _proc_query_sql(self, job_resp):
        """

        :param ResponseJobInfo job_resp:
        :return:
        """

        # If the server return result rows in submit job resp,
        # this query is run in Sync Mode
        if job_resp.rows and job_resp.schema:
            return SqlJob(job_resp.job_id, job_resp.job_type,
                          self, job_resp.schema, job_resp.rows, status='FINISHED')
        else:
            job_status = self.wait_for_finish(job_resp.job_id)
            return SqlJob(job_resp.job_id, job_resp.job_type, self, status=job_status)

    def _proc_ddl_dcl_sql(self, job_resp):
        """

        :param ResponseJobInfo job_resp:
        :return: SqlJob
        """

        # if not job_resp.schema or not job_resp.rows:
        #     message = 'DLI server response body error, ' \
        #               'schema or rows is None, job id: {0}'.format(job_resp.job_id)
        #     raise DliException(msg=message)

        return SqlJob(job_resp.job_id, job_resp.job_type,
                      self, job_resp.schema, job_resp.rows, status='FINISHED')

    def wait_for_finish(self, job_id, retry_times=1000):
        """

        :param str job_id: ID of a job returned after a job is submitted.
        :return: str job_status: The status of this job.
        """

        kwargs = {
            'project_id': self.dli_info.project_id,
            'x_auth_token': self.token,
            'job_id': job_id
        }

        loop_count = 0
        while True:
            if loop_count < 100:
                time.sleep(0.5)
            else:
                time.sleep(3)

            try:
                resp = self.rest_api.get_job_status(**kwargs)
            except ApiException as e:
                raise DliException(api_exception=e)
            if not resp.is_success:
                raise DliException(msg=resp.message)

            if resp.status == 'FAILED':
                message = 'Get job status, Job run failed.\n' \
                          'Job id = {0}\nDLI response: {1}'.format(resp.job_id,
                                                                   resp.message)
                raise DliException(msg=message)
            elif resp.status == 'FINISHED' or resp.status == 'CANCELLED':
                break

            # The sql is running too long times
            if loop_count > retry_times:
                message = 'Job run too long time, Job id = ' + resp.job_id
                raise DliException(msg=message)

            loop_count += 1

        return resp.status

    def export_table(self, table_name, db_name, data_path, queue_name='default', compress='none',
                     export_mode='ErrorIfExists'):
        """
        This method is used to export data to a file. You can export data from DLI
        tables to files. Currently, data can be exported only to OBS, and the OBS
        path must be specified to the folder level.
        This method is synchronous, blocking until execution is completed.

        :param str table_name: Name of the table from which data is exported.
        :param str db_name: Name of the database where the table from.
        :param str data_path: Path for storing the exported data (currently,
                            data can be stored on an OBS path starting with s3a only).
        :param str queue_name: Name of the queue that is specified to execute a task.
        :param str compress: Compression mode for exported data. Currently, the
                            compression modes gzip, bzip2, and deflate are supported. If you
                            do not want to compress data, enter none.
        :param str export_mode: Export mode. The parameter value can be ErrorIfExists
                            or Overwrite. If export_mode is not specified, this parameter
                            is set to ErrorIfExists by default.
        :return: tuple (job_id, status)
                    str job_id: ID of a job returned after a job is submitted.
                    str status: The status of this job.
        """

        exp_info = ExportJobRequest(data_path=data_path, data_type='csv',
                                    compress=compress, queue_name=queue_name,
                                    export_mode=export_mode, database_name=db_name,
                                    table_name=table_name)
        kwargs = {
            'project_id': self.dli_info.project_id,
            'x_auth_token': self.token,
            'export_info': exp_info
        }

        try:
            resp = self.rest_api.export_data(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        status = self.wait_for_finish(resp.job_id)
        return resp.job_id, status

    def import_table(self, table_name, db_name, data_path, data_type,
                     queue_name='default', options=None):
        """
        This method is used to import data from a file to a DLI table. Currently,
        only OBS data can be imported to DLI.
        This method is synchronous, blocking until execution is completed.

        :param str table_name: Name of the table to which data is imported.
        :param str db_name: Name of the database where the table to which data
                            is imported resides.
        :param str data_path: Path to which data is imported. Currently, only data
                            on an OBS path starting with s3a can be imported.
        :param str data_type: Type of the data to be imported. Currently, data types
                            CSV, Parquet, ORC, and JSON are supported.
        :param str queue_name: Name of the queue that is specified to execute a task.
                            If no queue is specified, the default queue is used.
        :param dict options: The dict options of the data type, for the detailed information refer to
                            https://support.huaweicloud.com/api-uquery/uquery_02_0100.html
                            For examples:
                            {
                                "with_column_header": true,
                                "delimiter": ",",
                                "quote_char": "\"",
                                "escape_char": "\\",
                                "date_format": "yyyy/MM/dd",
                                "timestamp_format": "yyyy/MM/dd hh:mm:ss"
                            }
        :return: tuple (job_id, status)
                    str job_id: ID of a job returned after a job is submitted.
                    str status: The status of this job.
        """

        delimiter = None
        quote_char = None
        escape_char = None
        date_format = None
        timestamp_format = None
        with_column_header = None

        if options:
            delimiter = options.get('delimiter')
            quote_char = options.get('quote_char')
            escape_char = options.get('escape_char')
            date_format = options.get('date_format')
            timestamp_format = options.get('timestamp_format')
            with_column_header = options.get('with_column_header')

        import_info = ImportJobRequest(data_path=data_path, data_type=data_type,
                                       table_name=table_name, with_column_header=with_column_header,
                                       delimiter=delimiter, quote_char=quote_char,
                                       escape_char=escape_char, date_format=date_format,
                                       timestamp_format=timestamp_format,
                                       database_name=db_name, queue_name=queue_name)
        kwargs = {
            'project_id': self.dli_info.project_id,
            'x_auth_token': self.token,
            'import_export_info': import_info
        }

        try:
            resp = self.rest_api.import_data(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        status = self.wait_for_finish(resp.job_id)
        return resp.job_id, status

    def submit_spark_batch_job(self, queue_name, batch_job_info):
        """

        Submit spark batch job
        This method is used to submit spark batch job to the server cluster.
        This method is a synchronous mode, blocking until spark batch job is completed.

        :param str queue_name:
        :param SparkJobInfo batch_job_info:
        :return: SparkBatchJob
        """

        batch_job_body = batch_job_info.get_request_body(queue_name)

        try:
            resp = self.rest_api.create_batch(project_id=self.dli_info.project_id,
                                              body=batch_job_body,
                                              x_auth_token=self.token)
        except ApiException as e:
            raise DliException(api_exception=e)

        spark_batch_job = SparkBatchJob(self, resp)

        return spark_batch_job

    def del_spark_batch_job(self, batch_id):
        """

        :param str batch_id: ID of a spark batch job returned after a spark batch job is submitted.
        :return:
        """

        try:
            if not batch_id:
                raise DliException(msg="No spark batch job in progress, cancel spark batch job failed")
            resp = self._rest_api.del_batch(self.dli_info.project_id,
                                             batch_id, x_auth_token=self.token)
        except ApiException as e:
            raise DliException(api_exception=e)
        return resp.msg

    def list_resources(self):
        """
        This method is used to list all resources of the project.

        :return: Resources
        """
        try:
            resp = self._rest_api.list_resources(self.dli_info.project_id,
                                                  x_auth_token=self.token)
        except ApiException as e:
            raise DliException(api_exception=e)

        pkg_resources = []
        for resources_info in resp.resources:
            pkg_resources.append(PackageResource(resources_info))

        group_resources = []
        for group_resource in resp.groups:
            group_resources.append(GroupResource(group_resource))

        resources = Resources(pkg_resources, group_resources)

        return resources

    def get_package_resource(self, resource_name, group_name=None):
        """
        This method is used to list user-specified package resource of the project

        :param str group_name: name of user defined group
        :param str resource_name: name of package resource
        :return: PackageResource
        """

        kwargs = {
            'project_id': self.dli_info.project_id,
            'x_auth_token': self.token,
            'resource_name': resource_name
        }

        if group_name is not None:
            kwargs['group'] = group_name

        try:
            resp = self._rest_api.get_resource_info(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)

        pkg_resource = PackageResource(resp)

        return pkg_resource

    def get_group_resource(self, group_name):
        """
        This method is used to list user-specified group resource of the project

        :param str group_name: name of user defined group
        :return: GroupResource
        """
        try:
            resp = self._rest_api.list_resources(self.dli_info.project_id,
                                                 x_auth_token=self.token)
        except ApiException as e:
            raise DliException(api_exception=e)

        group_resource = None
        for group_resource_info in resp.groups:
            if group_resource_info.group_name == group_name:
                group_resource = GroupResource(group_resource_info)
                break

        return group_resource

    def upload_resource_async(self, kind, paths, group=None):
        """
        This method is async upload user resources to server.
        This method will return resource names

        :param str kind: jar, py, file
        :param list[str] paths:
        :param str group:
        :return:
        """

        pkg_request_body = PkgResourceRequest(kind=kind, paths=paths, group=group, is_async=True)

        kwargs = {
            'project_id': self.dli_info.project_id,
            'body': pkg_request_body,
            'x_auth_token': self.token,
        }

        try:
            resourceMsg = self._rest_api.upload_resources(**kwargs)
            if group is None :
               return resourceMsg.resource_names
            else :
               return resourceMsg.resources

        except ApiException as e:
            raise DliException(api_exception=e)


    def upload_resource(self, kind, paths, group=None):
        """
        This method is upload user resouces to server.

        :param str kind: jar, py, file
        :param list[str] paths:
        :param str group:
        :return:
        """

        pkg_request_body = PkgResourceRequest(kind=kind, paths=paths, group=group)

        kwargs = {
            'project_id': self.dli_info.project_id,
            'body': pkg_request_body,
            'x_auth_token': self.token,
        }

        try:
            resp = self._rest_api.upload_resources(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)

        return

    def delete_resource(self, resource_name, group_name=None):
        """
        This method is used to delete user-specified package resource of the project

        :param str resource_name: name of package resource
        :param str group_name: name of user defined group

        :return:
        """

        kwargs = {
            'project_id': self.dli_info.project_id,
            'x_auth_token': self.token,
            'resource_name': resource_name
        }

        if group_name is not None:
            kwargs['group'] = group_name

        try:
            resp = self._rest_api.delete_resource(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)

        return
