#!/usr/bin/python
# -*- coding: UTF-8 -*-

from dli.exception import DliException
from dli.logger import logger
from dli.reader import AvroObsReader, JsonObsReader
from dli.row import Row
from dli.table import TableSchema

import uuid


class DownloadJob(object):

    def __init__(self, client, queue_name, db_name, table_name, part_spec=None, job=None):
        """
        :param client: the DLIClient
        :param queue_name: queue name, use this queue for computing
        :param db_name: the database name which the data to download
        :param table_name: the table name which the data to download
        :param part_spec: the partition spec for a partitioned table
        :param job: the sql job
        """

        self._client = client
        if job is not None:
            self._job = job
            self._file_format = "json"
            # SqlJob.get_schema() return is a list of Column
            self._table_schema = TableSchema("", job.get_schema(queue_name))
            self._prepare(self._table_schema)
            data_path = "obs://%s/%s" % (self._bucket_name, self._object_prefix)
            job_id = job.export_result(data_path=data_path, data_type='json', queue_name=queue_name, _async=True)
            self._job.set_job_id(job_id)
            self._has_rename_col = False
        else:
            self._file_format = "avro"
            self._table_schema = self._client.get_table_schema(db_name, table_name)
            self._prepare(self._table_schema, db_name, table_name)
            select_cols = []
            i = 0
            for col in self._table_schema.columns:
                select_cols.append("`%s` AS `c%d`" % (col.name, i))
                i += 1
            part_spec_clause = []
            if part_spec is not None:
                part_specs = part_spec.split(",")
                for part_spec_kv in part_specs:
                    kv = part_spec_kv.split("=")
                    part_spec_clause.append("`%s` = %s" % (kv[0], kv[1]))
            if len(part_spec_clause) != 0:
                export_sql = "INSERT OVERWRITE DIRECTORY 'obs://%s/%s' USING avro SELECT %s FROM `%s`.`%s` WHERE %s" % \
                             (self._bucket_name, self._object_prefix, ', '.join(select_cols), db_name, table_name,
                              ' AND '.join(part_spec_clause))
            else:
                export_sql = "INSERT OVERWRITE DIRECTORY 'obs://%s/%s' USING avro SELECT %s FROM `%s`.`%s`" % \
                             (self._bucket_name, self._object_prefix, ', '.join(select_cols), db_name, table_name)
            self._job = client.async_execute_sql(sql=export_sql, db_name=db_name, queue_name=queue_name)
            self._has_rename_col = True

        self._is_ready = False

    def _prepare(self, table_schema, db_name=None, tbl_name=None):
        part_cols = []
        for col in table_schema.columns:
            if col.is_partition_column:
                part_cols.append(col)
        self._bucket_name = self._create_tunnel()
        if db_name is None or tbl_name is None:
            self._object_prefix = "export/exportDb/exportTbl/%s" % str(uuid.uuid1())
        else:
            self._object_prefix = "download/%s/%s/%s" % (db_name, tbl_name, str(uuid.uuid1()))

    def _create_tunnel(self):
        empty_body = {'job_id': 'unused'}
        kwargs = {
            'project_id': self._client.dli_info.project_id,
            'x_auth_token': self._client.token,
            'empty_body': empty_body
        }
        resp = self._client.rest_api.create_download_job(**kwargs)
        return resp.bucket_name

    def create_reader(self):
        if not self._is_ready:
            raise DliException(msg="There is not ready for create reader, please check status first.")
        dli_info = self._client.dli_info
        if self._file_format == "avro":
            return AvroObsReader(self._table_schema, dli_info.obs, dli_info.ak, dli_info.sk,
                                 self._bucket_name, self._object_prefix, self._has_rename_col)
        elif self._file_format == "json":
            return JsonObsReader(self._table_schema, dli_info.obs, dli_info.ak, dli_info.sk,
                                 self._bucket_name, self._object_prefix, self._has_rename_col)
        else:
            raise DliException(msg="Unsupported operator")

    def get_download_status(self):
        if self._job is None:
            raise DliException(msg="There is not job have been created, please check.")
        status = self._job.get_status()
        logger.info("download job internal status: %s" % status)
        if status == "FINISHED":
            self._is_ready = True
        return status

    def new_empty_row(self):
        return Row(self._table_schema, [])
