#!/usr/bin/python
# -*- coding: UTF-8 -*-

from avro.datafile import DataFileReader
from avro.io import DatumReader
from datetime import datetime
from dli.exception import DliException
from dli.logger import logger
from dli.row import Row
from obs.client import ObsClient

import json
import os
import time


class Reader(object):
    def __init__(self, table_schema, obs_endpoint, ak, sk, bucket_name, obj_prefix, has_rename_col=False):
        self._table_schema = table_schema
        self._obs_client = ObsClient(server=obs_endpoint, access_key_id=ak, secret_access_key=sk)
        self._bucket_name = bucket_name
        self._object_prefix = obj_prefix
        self._obs_objects = []
        self._list_obs_object()
        self._pre_tmp_file = None
        self._total_iter = len(self._obs_objects)
        self._next_iter_idx = 0
        self._current_row = None
        self._has_rename_col = has_rename_col

    def __iter__(self):
        return ObsReaderIterator(self)

    def _list_obs_object(self):
        next_marker = None
        while True:
            resp = self._obs_client.listObjects(bucketName=self._bucket_name, prefix=self._object_prefix,
                                                max_keys=1000, marker=next_marker)
            for content in resp.body.contents:
                if content.size > 0:
                    self._obs_objects.append(content.key)
            if not resp.body.is_truncated:
                break
            next_marker = resp.body.next_marker
        logger.info("object size: %d" % len(self._obs_objects))

    def next(self):
        pass

    def fetch(self):
        return self._current_row


class ObsReaderIterator(object):
    def __init__(self, obs_reader):
        self._obs_reader = obs_reader

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def next(self):
        try:
            ret = self._obs_reader.next()
            if ret is not True:
                raise StopIteration()
            return self._obs_reader.fetch()
        except IndexError:
            raise StopIteration()


class AvroObsReader(Reader):
    def __init__(self, table_schema, obs_endpoint, ak, sk, bucket_name, obj_prefix, has_rename_col=False):
        super().__init__(table_schema, obs_endpoint, ak, sk, bucket_name, obj_prefix, has_rename_col)
        self._iter = self._get_next_iter()

    def _get_next_iter(self):
        if self._total_iter > 0 and self._next_iter_idx < self._total_iter:
            return self._create_iter()
        else:
            return None

    def _create_iter(self):
        # cleanup tmp file
        if self._pre_tmp_file is not None:
            os.remove(self._pre_tmp_file)

        curr_iter_idx = self._next_iter_idx
        self._next_iter_idx += 1
        object_key = self._obs_objects[curr_iter_idx]
        logger.info("create iterator for object: %s" % object_key)
        """
           TODO: read from stream rather than download data to local
        """
        local_path = "/tmp/%s" % object_key
        resp = self._obs_client.getObject(bucketName=self._bucket_name, objectKey=object_key, downloadPath=local_path)
        if resp.status < 300:
            self._pre_tmp_file = local_path
            return DataFileReader(open(local_path, "rb"), DatumReader())
        else:
            raise DliException(msg="read data from obs failed")

    def next(self):
        if self._iter is not None:
            try:
                self._get_row()
                return True
            except StopIteration:
                self._iter.close()
                if self._next_iter_idx < self._total_iter:
                    self._iter = self._get_next_iter()
                    self._get_row()
                    return True
        else:
            logger.info("iter didn't init, create it")
            if self._next_iter_idx < self._total_iter:
                self._iter = self._get_next_iter()
                self._get_row()
                return True
        # cleanup tmp file
        if self._pre_tmp_file is not None:
            os.remove(self._pre_tmp_file)
        if self._iter is not None:
            self._iter.close()
        return False

    def _get_row(self):
        row_data = list(self._iter.next().values())
        data_schema = json.loads(self._iter.meta.get("avro.schema")).get("fields")
        sql_cols = self._table_schema.columns
        for i in range(0, len(row_data)):
            if row_data[i] is not None:
                self._update_col(row_data, i, sql_cols[i].type, data_schema[i].get("type")[0])
        self._current_row = Row(sql_cols, row_data)

    @staticmethod
    def _update_col(data_list, index, data_type, avro_type):
        # TODO: handle decimal, date and timestamp type for spark-avro from Spark 2.4
        if data_type == "date" and avro_type == "long":
            tm = int(data_list[index]) / 1000
            data_list[index] = datetime.fromtimestamp(tm).strftime('%Y-%m-%d')


class JsonObsReader(Reader):
    def __init__(self, table_schema, obs_endpoint, ak, sk, bucket_name, obj_prefix, has_rename_col=False):
        super().__init__(table_schema, obs_endpoint, ak, sk, bucket_name, obj_prefix, has_rename_col)
        self._iter = self._get_next_iter()
        if self._iter is not None:
            self._next_line = self._iter.readline()

    def _get_next_iter(self):
        if self._total_iter > 0 and self._next_iter_idx < self._total_iter:
            return self._create_iter()
        else:
            return None

    def _create_iter(self):
        # cleanup tmp file
        if self._pre_tmp_file is not None:
            os.remove(self._pre_tmp_file)

        curr_iter_idx = self._next_iter_idx
        self._next_iter_idx += 1
        object_key = self._obs_objects[curr_iter_idx]
        logger.info("create iterator for object: %s" % object_key)
        """
           TODO: read from stream rather than download data to local
        """
        local_path = "/tmp/%s" % object_key
        resp = self._obs_client.getObject(bucketName=self._bucket_name, objectKey=object_key, downloadPath=local_path)
        if resp.status < 300:
            self._pre_tmp_file = local_path
            return open(local_path, "rb")
        else:
            raise DliException(msg="read data from obs failed")

    def next(self):
        if self._iter is not None:
            if len(self._next_line) != 0:
                self._get_row()
                self._next_line = self._iter.readline()
                return True
            else:  # EOF
                self._iter.close()
                if self._next_iter_idx < self._total_iter:
                    self._iter = self._get_next_iter()
                    if self._iter is not None:
                        self._next_line = self._iter.readline()
                        self._get_row()
                        self._next_line = self._iter.readline()
                        return True
        else:
            logger.info("iter didn't init, create it")
            if self._next_iter_idx < self._total_iter:
                self._iter = self._get_next_iter()
                if self._iter is not None:
                    self._next_line = self._iter.readline()
                    self._get_row()
                    self._next_line = self._iter.readline()
                    return True
        # cleanup tmp file
        if self._pre_tmp_file is not None:
            os.remove(self._pre_tmp_file)
        if self._iter is not None:
            self._iter.close()
        return False

    def _get_row(self):
        sql_cols = self._table_schema.columns
        row_data = self._update_row(json.loads(self._next_line), sql_cols)
        self._current_row = Row(sql_cols, row_data)

    @staticmethod
    def _update_row(data_dict, sql_cols):
        data_list = []
        for col in sql_cols:
            data = data_dict.get(col.name, None)
            if data is None:
                data_list.append(None)
                continue

            if col.type == "timestamp":
                ts_int = time.mktime(datetime.strptime(data, '%Y-%m-%dT%H:%M:%S.%f%z').timetuple())
                ts_int *= 1000
                data_list.append(ts_int)
            else:
                data_list.append(data)
        return data_list
